# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ListenerArgs', 'Listener']

@pulumi.input_type
class ListenerArgs:
    def __init__(__self__, *,
                 default_backend_set_name: pulumi.Input[str],
                 load_balancer_id: pulumi.Input[str],
                 port: pulumi.Input[int],
                 protocol: pulumi.Input[str],
                 connection_configuration: Optional[pulumi.Input['ListenerConnectionConfigurationArgs']] = None,
                 hostname_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 path_route_set_name: Optional[pulumi.Input[str]] = None,
                 routing_policy_name: Optional[pulumi.Input[str]] = None,
                 rule_set_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssl_configuration: Optional[pulumi.Input['ListenerSslConfigurationArgs']] = None):
        """
        The set of arguments for constructing a Listener resource.
        :param pulumi.Input[str] default_backend_set_name: (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        :param pulumi.Input[str] load_balancer_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        :param pulumi.Input[int] port: (Updatable) The communication port for the listener.  Example: `80`
        :param pulumi.Input[str] protocol: (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        :param pulumi.Input['ListenerConnectionConfigurationArgs'] connection_configuration: (Updatable) Configuration details for the connection between the client and backend servers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] hostname_names: (Updatable) An array of hostname resource names.
        :param pulumi.Input[str] name: A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        :param pulumi.Input[str] path_route_set_name: (Updatable) Deprecated. Please use `routingPolicies` instead.
               
               The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.
               
               Example: `example_path_route_set`
        :param pulumi.Input[str] routing_policy_name: (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] rule_set_names: (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        :param pulumi.Input['ListenerSslConfigurationArgs'] ssl_configuration: (Updatable) The load balancer's SSL handling configuration details.
               
               **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        pulumi.set(__self__, "default_backend_set_name", default_backend_set_name)
        pulumi.set(__self__, "load_balancer_id", load_balancer_id)
        pulumi.set(__self__, "port", port)
        pulumi.set(__self__, "protocol", protocol)
        if connection_configuration is not None:
            pulumi.set(__self__, "connection_configuration", connection_configuration)
        if hostname_names is not None:
            pulumi.set(__self__, "hostname_names", hostname_names)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if path_route_set_name is not None:
            pulumi.set(__self__, "path_route_set_name", path_route_set_name)
        if routing_policy_name is not None:
            pulumi.set(__self__, "routing_policy_name", routing_policy_name)
        if rule_set_names is not None:
            pulumi.set(__self__, "rule_set_names", rule_set_names)
        if ssl_configuration is not None:
            pulumi.set(__self__, "ssl_configuration", ssl_configuration)

    @property
    @pulumi.getter(name="defaultBackendSetName")
    def default_backend_set_name(self) -> pulumi.Input[str]:
        """
        (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        """
        return pulumi.get(self, "default_backend_set_name")

    @default_backend_set_name.setter
    def default_backend_set_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "default_backend_set_name", value)

    @property
    @pulumi.getter(name="loadBalancerId")
    def load_balancer_id(self) -> pulumi.Input[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        """
        return pulumi.get(self, "load_balancer_id")

    @load_balancer_id.setter
    def load_balancer_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "load_balancer_id", value)

    @property
    @pulumi.getter
    def port(self) -> pulumi.Input[int]:
        """
        (Updatable) The communication port for the listener.  Example: `80`
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: pulumi.Input[int]):
        pulumi.set(self, "port", value)

    @property
    @pulumi.getter
    def protocol(self) -> pulumi.Input[str]:
        """
        (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: pulumi.Input[str]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="connectionConfiguration")
    def connection_configuration(self) -> Optional[pulumi.Input['ListenerConnectionConfigurationArgs']]:
        """
        (Updatable) Configuration details for the connection between the client and backend servers.
        """
        return pulumi.get(self, "connection_configuration")

    @connection_configuration.setter
    def connection_configuration(self, value: Optional[pulumi.Input['ListenerConnectionConfigurationArgs']]):
        pulumi.set(self, "connection_configuration", value)

    @property
    @pulumi.getter(name="hostnameNames")
    def hostname_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) An array of hostname resource names.
        """
        return pulumi.get(self, "hostname_names")

    @hostname_names.setter
    def hostname_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "hostname_names", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="pathRouteSetName")
    def path_route_set_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) Deprecated. Please use `routingPolicies` instead.

        The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.

        Example: `example_path_route_set`
        """
        return pulumi.get(self, "path_route_set_name")

    @path_route_set_name.setter
    def path_route_set_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "path_route_set_name", value)

    @property
    @pulumi.getter(name="routingPolicyName")
    def routing_policy_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        """
        return pulumi.get(self, "routing_policy_name")

    @routing_policy_name.setter
    def routing_policy_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "routing_policy_name", value)

    @property
    @pulumi.getter(name="ruleSetNames")
    def rule_set_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        """
        return pulumi.get(self, "rule_set_names")

    @rule_set_names.setter
    def rule_set_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "rule_set_names", value)

    @property
    @pulumi.getter(name="sslConfiguration")
    def ssl_configuration(self) -> Optional[pulumi.Input['ListenerSslConfigurationArgs']]:
        """
        (Updatable) The load balancer's SSL handling configuration details.

        **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        return pulumi.get(self, "ssl_configuration")

    @ssl_configuration.setter
    def ssl_configuration(self, value: Optional[pulumi.Input['ListenerSslConfigurationArgs']]):
        pulumi.set(self, "ssl_configuration", value)


@pulumi.input_type
class _ListenerState:
    def __init__(__self__, *,
                 connection_configuration: Optional[pulumi.Input['ListenerConnectionConfigurationArgs']] = None,
                 default_backend_set_name: Optional[pulumi.Input[str]] = None,
                 hostname_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 load_balancer_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 path_route_set_name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 routing_policy_name: Optional[pulumi.Input[str]] = None,
                 rule_set_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssl_configuration: Optional[pulumi.Input['ListenerSslConfigurationArgs']] = None,
                 state: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Listener resources.
        :param pulumi.Input['ListenerConnectionConfigurationArgs'] connection_configuration: (Updatable) Configuration details for the connection between the client and backend servers.
        :param pulumi.Input[str] default_backend_set_name: (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] hostname_names: (Updatable) An array of hostname resource names.
        :param pulumi.Input[str] load_balancer_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        :param pulumi.Input[str] name: A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        :param pulumi.Input[str] path_route_set_name: (Updatable) Deprecated. Please use `routingPolicies` instead.
               
               The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.
               
               Example: `example_path_route_set`
        :param pulumi.Input[int] port: (Updatable) The communication port for the listener.  Example: `80`
        :param pulumi.Input[str] protocol: (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        :param pulumi.Input[str] routing_policy_name: (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] rule_set_names: (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        :param pulumi.Input['ListenerSslConfigurationArgs'] ssl_configuration: (Updatable) The load balancer's SSL handling configuration details.
               
               **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        if connection_configuration is not None:
            pulumi.set(__self__, "connection_configuration", connection_configuration)
        if default_backend_set_name is not None:
            pulumi.set(__self__, "default_backend_set_name", default_backend_set_name)
        if hostname_names is not None:
            pulumi.set(__self__, "hostname_names", hostname_names)
        if load_balancer_id is not None:
            pulumi.set(__self__, "load_balancer_id", load_balancer_id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if path_route_set_name is not None:
            pulumi.set(__self__, "path_route_set_name", path_route_set_name)
        if port is not None:
            pulumi.set(__self__, "port", port)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)
        if routing_policy_name is not None:
            pulumi.set(__self__, "routing_policy_name", routing_policy_name)
        if rule_set_names is not None:
            pulumi.set(__self__, "rule_set_names", rule_set_names)
        if ssl_configuration is not None:
            pulumi.set(__self__, "ssl_configuration", ssl_configuration)
        if state is not None:
            pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter(name="connectionConfiguration")
    def connection_configuration(self) -> Optional[pulumi.Input['ListenerConnectionConfigurationArgs']]:
        """
        (Updatable) Configuration details for the connection between the client and backend servers.
        """
        return pulumi.get(self, "connection_configuration")

    @connection_configuration.setter
    def connection_configuration(self, value: Optional[pulumi.Input['ListenerConnectionConfigurationArgs']]):
        pulumi.set(self, "connection_configuration", value)

    @property
    @pulumi.getter(name="defaultBackendSetName")
    def default_backend_set_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        """
        return pulumi.get(self, "default_backend_set_name")

    @default_backend_set_name.setter
    def default_backend_set_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "default_backend_set_name", value)

    @property
    @pulumi.getter(name="hostnameNames")
    def hostname_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) An array of hostname resource names.
        """
        return pulumi.get(self, "hostname_names")

    @hostname_names.setter
    def hostname_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "hostname_names", value)

    @property
    @pulumi.getter(name="loadBalancerId")
    def load_balancer_id(self) -> Optional[pulumi.Input[str]]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        """
        return pulumi.get(self, "load_balancer_id")

    @load_balancer_id.setter
    def load_balancer_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "load_balancer_id", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="pathRouteSetName")
    def path_route_set_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) Deprecated. Please use `routingPolicies` instead.

        The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.

        Example: `example_path_route_set`
        """
        return pulumi.get(self, "path_route_set_name")

    @path_route_set_name.setter
    def path_route_set_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "path_route_set_name", value)

    @property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[int]]:
        """
        (Updatable) The communication port for the listener.  Example: `80`
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "port", value)

    @property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="routingPolicyName")
    def routing_policy_name(self) -> Optional[pulumi.Input[str]]:
        """
        (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        """
        return pulumi.get(self, "routing_policy_name")

    @routing_policy_name.setter
    def routing_policy_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "routing_policy_name", value)

    @property
    @pulumi.getter(name="ruleSetNames")
    def rule_set_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        """
        return pulumi.get(self, "rule_set_names")

    @rule_set_names.setter
    def rule_set_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "rule_set_names", value)

    @property
    @pulumi.getter(name="sslConfiguration")
    def ssl_configuration(self) -> Optional[pulumi.Input['ListenerSslConfigurationArgs']]:
        """
        (Updatable) The load balancer's SSL handling configuration details.

        **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        return pulumi.get(self, "ssl_configuration")

    @ssl_configuration.setter
    def ssl_configuration(self, value: Optional[pulumi.Input['ListenerSslConfigurationArgs']]):
        pulumi.set(self, "ssl_configuration", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)


class Listener(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connection_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerConnectionConfigurationArgs']]] = None,
                 default_backend_set_name: Optional[pulumi.Input[str]] = None,
                 hostname_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 load_balancer_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 path_route_set_name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 routing_policy_name: Optional[pulumi.Input[str]] = None,
                 rule_set_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssl_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerSslConfigurationArgs']]] = None,
                 __props__=None):
        """
        This resource provides the Listener resource in Oracle Cloud Infrastructure Load Balancer service.

        Adds a listener to a load balancer.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_listener = oci.load_balancer.Listener("testListener",
            default_backend_set_name=oci_load_balancer_backend_set["test_backend_set"]["name"],
            load_balancer_id=oci_load_balancer_load_balancer["test_load_balancer"]["id"],
            port=var["listener_port"],
            protocol=var["listener_protocol"],
            connection_configuration=oci.load_balancer.ListenerConnectionConfigurationArgs(
                idle_timeout_in_seconds=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                backend_tcp_proxy_protocol_version=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            hostname_names=[oci_load_balancer_hostname["test_hostname"]["name"]],
            path_route_set_name=oci_load_balancer_path_route_set["test_path_route_set"]["name"],
            routing_policy_name=oci_load_balancer_load_balancer_routing_policy["test_load_balancer_routing_policy"]["name"],
            rule_set_names=[oci_load_balancer_rule_set["test_rule_set"]["name"]],
            ssl_configuration=oci.load_balancer.ListenerSslConfigurationArgs(
                certificate_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                certificate_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                cipher_suite_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                protocols=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                server_order_preference=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                trusted_certificate_authority_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                verify_depth=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                verify_peer_certificate=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ))
        ```

        ## Import

        Listeners can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:LoadBalancer/listener:Listener test_listener "loadBalancers/{loadBalancerId}/listeners/{listenerName}"
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[pulumi.InputType['ListenerConnectionConfigurationArgs']] connection_configuration: (Updatable) Configuration details for the connection between the client and backend servers.
        :param pulumi.Input[str] default_backend_set_name: (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] hostname_names: (Updatable) An array of hostname resource names.
        :param pulumi.Input[str] load_balancer_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        :param pulumi.Input[str] name: A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        :param pulumi.Input[str] path_route_set_name: (Updatable) Deprecated. Please use `routingPolicies` instead.
               
               The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.
               
               Example: `example_path_route_set`
        :param pulumi.Input[int] port: (Updatable) The communication port for the listener.  Example: `80`
        :param pulumi.Input[str] protocol: (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        :param pulumi.Input[str] routing_policy_name: (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] rule_set_names: (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        :param pulumi.Input[pulumi.InputType['ListenerSslConfigurationArgs']] ssl_configuration: (Updatable) The load balancer's SSL handling configuration details.
               
               **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ListenerArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource provides the Listener resource in Oracle Cloud Infrastructure Load Balancer service.

        Adds a listener to a load balancer.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_oci as oci

        test_listener = oci.load_balancer.Listener("testListener",
            default_backend_set_name=oci_load_balancer_backend_set["test_backend_set"]["name"],
            load_balancer_id=oci_load_balancer_load_balancer["test_load_balancer"]["id"],
            port=var["listener_port"],
            protocol=var["listener_protocol"],
            connection_configuration=oci.load_balancer.ListenerConnectionConfigurationArgs(
                idle_timeout_in_seconds=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                backend_tcp_proxy_protocol_version=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ),
            hostname_names=[oci_load_balancer_hostname["test_hostname"]["name"]],
            path_route_set_name=oci_load_balancer_path_route_set["test_path_route_set"]["name"],
            routing_policy_name=oci_load_balancer_load_balancer_routing_policy["test_load_balancer_routing_policy"]["name"],
            rule_set_names=[oci_load_balancer_rule_set["test_rule_set"]["name"]],
            ssl_configuration=oci.load_balancer.ListenerSslConfigurationArgs(
                certificate_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                certificate_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                cipher_suite_name=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                protocols=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                server_order_preference=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                trusted_certificate_authority_ids=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                verify_depth=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
                verify_peer_certificate=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
            ))
        ```

        ## Import

        Listeners can be imported using the `id`, e.g.

        ```sh
         $ pulumi import oci:LoadBalancer/listener:Listener test_listener "loadBalancers/{loadBalancerId}/listeners/{listenerName}"
        ```

        :param str resource_name: The name of the resource.
        :param ListenerArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ListenerArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connection_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerConnectionConfigurationArgs']]] = None,
                 default_backend_set_name: Optional[pulumi.Input[str]] = None,
                 hostname_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 load_balancer_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 path_route_set_name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 routing_policy_name: Optional[pulumi.Input[str]] = None,
                 rule_set_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 ssl_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerSslConfigurationArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ListenerArgs.__new__(ListenerArgs)

            __props__.__dict__["connection_configuration"] = connection_configuration
            if default_backend_set_name is None and not opts.urn:
                raise TypeError("Missing required property 'default_backend_set_name'")
            __props__.__dict__["default_backend_set_name"] = default_backend_set_name
            __props__.__dict__["hostname_names"] = hostname_names
            if load_balancer_id is None and not opts.urn:
                raise TypeError("Missing required property 'load_balancer_id'")
            __props__.__dict__["load_balancer_id"] = load_balancer_id
            __props__.__dict__["name"] = name
            __props__.__dict__["path_route_set_name"] = path_route_set_name
            if port is None and not opts.urn:
                raise TypeError("Missing required property 'port'")
            __props__.__dict__["port"] = port
            if protocol is None and not opts.urn:
                raise TypeError("Missing required property 'protocol'")
            __props__.__dict__["protocol"] = protocol
            __props__.__dict__["routing_policy_name"] = routing_policy_name
            __props__.__dict__["rule_set_names"] = rule_set_names
            __props__.__dict__["ssl_configuration"] = ssl_configuration
            __props__.__dict__["state"] = None
        super(Listener, __self__).__init__(
            'oci:LoadBalancer/listener:Listener',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            connection_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerConnectionConfigurationArgs']]] = None,
            default_backend_set_name: Optional[pulumi.Input[str]] = None,
            hostname_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            load_balancer_id: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            path_route_set_name: Optional[pulumi.Input[str]] = None,
            port: Optional[pulumi.Input[int]] = None,
            protocol: Optional[pulumi.Input[str]] = None,
            routing_policy_name: Optional[pulumi.Input[str]] = None,
            rule_set_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            ssl_configuration: Optional[pulumi.Input[pulumi.InputType['ListenerSslConfigurationArgs']]] = None,
            state: Optional[pulumi.Input[str]] = None) -> 'Listener':
        """
        Get an existing Listener resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[pulumi.InputType['ListenerConnectionConfigurationArgs']] connection_configuration: (Updatable) Configuration details for the connection between the client and backend servers.
        :param pulumi.Input[str] default_backend_set_name: (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] hostname_names: (Updatable) An array of hostname resource names.
        :param pulumi.Input[str] load_balancer_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        :param pulumi.Input[str] name: A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        :param pulumi.Input[str] path_route_set_name: (Updatable) Deprecated. Please use `routingPolicies` instead.
               
               The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.
               
               Example: `example_path_route_set`
        :param pulumi.Input[int] port: (Updatable) The communication port for the listener.  Example: `80`
        :param pulumi.Input[str] protocol: (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        :param pulumi.Input[str] routing_policy_name: (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        :param pulumi.Input[Sequence[pulumi.Input[str]]] rule_set_names: (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        :param pulumi.Input[pulumi.InputType['ListenerSslConfigurationArgs']] ssl_configuration: (Updatable) The load balancer's SSL handling configuration details.
               
               **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ListenerState.__new__(_ListenerState)

        __props__.__dict__["connection_configuration"] = connection_configuration
        __props__.__dict__["default_backend_set_name"] = default_backend_set_name
        __props__.__dict__["hostname_names"] = hostname_names
        __props__.__dict__["load_balancer_id"] = load_balancer_id
        __props__.__dict__["name"] = name
        __props__.__dict__["path_route_set_name"] = path_route_set_name
        __props__.__dict__["port"] = port
        __props__.__dict__["protocol"] = protocol
        __props__.__dict__["routing_policy_name"] = routing_policy_name
        __props__.__dict__["rule_set_names"] = rule_set_names
        __props__.__dict__["ssl_configuration"] = ssl_configuration
        __props__.__dict__["state"] = state
        return Listener(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="connectionConfiguration")
    def connection_configuration(self) -> pulumi.Output['outputs.ListenerConnectionConfiguration']:
        """
        (Updatable) Configuration details for the connection between the client and backend servers.
        """
        return pulumi.get(self, "connection_configuration")

    @property
    @pulumi.getter(name="defaultBackendSetName")
    def default_backend_set_name(self) -> pulumi.Output[str]:
        """
        (Updatable) The name of the associated backend set.  Example: `example_backend_set`
        """
        return pulumi.get(self, "default_backend_set_name")

    @property
    @pulumi.getter(name="hostnameNames")
    def hostname_names(self) -> pulumi.Output[Sequence[str]]:
        """
        (Updatable) An array of hostname resource names.
        """
        return pulumi.get(self, "hostname_names")

    @property
    @pulumi.getter(name="loadBalancerId")
    def load_balancer_id(self) -> pulumi.Output[str]:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the load balancer on which to add a listener.
        """
        return pulumi.get(self, "load_balancer_id")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        A friendly name for the listener. It must be unique and it cannot be changed. Avoid entering confidential information.  Example: `example_listener`
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="pathRouteSetName")
    def path_route_set_name(self) -> pulumi.Output[str]:
        """
        (Updatable) Deprecated. Please use `routingPolicies` instead.

        The name of the set of path-based routing rules, [PathRouteSet](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/PathRouteSet/), applied to this listener's traffic.

        Example: `example_path_route_set`
        """
        return pulumi.get(self, "path_route_set_name")

    @property
    @pulumi.getter
    def port(self) -> pulumi.Output[int]:
        """
        (Updatable) The communication port for the listener.  Example: `80`
        """
        return pulumi.get(self, "port")

    @property
    @pulumi.getter
    def protocol(self) -> pulumi.Output[str]:
        """
        (Updatable) The protocol on which the listener accepts connection requests. To get a list of valid protocols, use the [ListProtocols](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/LoadBalancerProtocol/ListProtocols) operation.  Example: `HTTP`
        """
        return pulumi.get(self, "protocol")

    @property
    @pulumi.getter(name="routingPolicyName")
    def routing_policy_name(self) -> pulumi.Output[str]:
        """
        (Updatable) The name of the routing policy applied to this listener's traffic.  Example: `example_routing_policy`
        """
        return pulumi.get(self, "routing_policy_name")

    @property
    @pulumi.getter(name="ruleSetNames")
    def rule_set_names(self) -> pulumi.Output[Sequence[str]]:
        """
        (Updatable) The names of the [rule sets](https://docs.cloud.oracle.com/iaas/api/#/en/loadbalancer/20170115/RuleSet/) to apply to the listener.  Example: ["example_rule_set"]
        """
        return pulumi.get(self, "rule_set_names")

    @property
    @pulumi.getter(name="sslConfiguration")
    def ssl_configuration(self) -> pulumi.Output[Optional['outputs.ListenerSslConfiguration']]:
        """
        (Updatable) The load balancer's SSL handling configuration details.

        **Warning:** Oracle recommends that you avoid using any confidential information when you supply string values using the API.
        """
        return pulumi.get(self, "ssl_configuration")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        return pulumi.get(self, "state")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetVmClusterNetworkResult',
    'AwaitableGetVmClusterNetworkResult',
    'get_vm_cluster_network',
    'get_vm_cluster_network_output',
]

@pulumi.output_type
class GetVmClusterNetworkResult:
    """
    A collection of values returned by getVmClusterNetwork.
    """
    def __init__(__self__, action=None, compartment_id=None, defined_tags=None, display_name=None, dns=None, exadata_infrastructure_id=None, freeform_tags=None, id=None, lifecycle_details=None, ntps=None, scans=None, state=None, time_created=None, validate_vm_cluster_network=None, vm_cluster_id=None, vm_cluster_network_id=None, vm_networks=None):
        if action and not isinstance(action, str):
            raise TypeError("Expected argument 'action' to be a str")
        pulumi.set(__self__, "action", action)
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if dns and not isinstance(dns, list):
            raise TypeError("Expected argument 'dns' to be a list")
        pulumi.set(__self__, "dns", dns)
        if exadata_infrastructure_id and not isinstance(exadata_infrastructure_id, str):
            raise TypeError("Expected argument 'exadata_infrastructure_id' to be a str")
        pulumi.set(__self__, "exadata_infrastructure_id", exadata_infrastructure_id)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if lifecycle_details and not isinstance(lifecycle_details, str):
            raise TypeError("Expected argument 'lifecycle_details' to be a str")
        pulumi.set(__self__, "lifecycle_details", lifecycle_details)
        if ntps and not isinstance(ntps, list):
            raise TypeError("Expected argument 'ntps' to be a list")
        pulumi.set(__self__, "ntps", ntps)
        if scans and not isinstance(scans, list):
            raise TypeError("Expected argument 'scans' to be a list")
        pulumi.set(__self__, "scans", scans)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if time_created and not isinstance(time_created, str):
            raise TypeError("Expected argument 'time_created' to be a str")
        pulumi.set(__self__, "time_created", time_created)
        if validate_vm_cluster_network and not isinstance(validate_vm_cluster_network, bool):
            raise TypeError("Expected argument 'validate_vm_cluster_network' to be a bool")
        pulumi.set(__self__, "validate_vm_cluster_network", validate_vm_cluster_network)
        if vm_cluster_id and not isinstance(vm_cluster_id, str):
            raise TypeError("Expected argument 'vm_cluster_id' to be a str")
        pulumi.set(__self__, "vm_cluster_id", vm_cluster_id)
        if vm_cluster_network_id and not isinstance(vm_cluster_network_id, str):
            raise TypeError("Expected argument 'vm_cluster_network_id' to be a str")
        pulumi.set(__self__, "vm_cluster_network_id", vm_cluster_network_id)
        if vm_networks and not isinstance(vm_networks, list):
            raise TypeError("Expected argument 'vm_networks' to be a list")
        pulumi.set(__self__, "vm_networks", vm_networks)

    @property
    @pulumi.getter
    def action(self) -> str:
        return pulumi.get(self, "action")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        The user-friendly name for the VM cluster network. The name does not need to be unique.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def dns(self) -> Sequence[str]:
        """
        The list of DNS server IP addresses. Maximum of 3 allowed.
        """
        return pulumi.get(self, "dns")

    @property
    @pulumi.getter(name="exadataInfrastructureId")
    def exadata_infrastructure_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the Exadata infrastructure.
        """
        return pulumi.get(self, "exadata_infrastructure_id")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the VM cluster network.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="lifecycleDetails")
    def lifecycle_details(self) -> str:
        """
        Additional information about the current lifecycle state.
        """
        return pulumi.get(self, "lifecycle_details")

    @property
    @pulumi.getter
    def ntps(self) -> Sequence[str]:
        """
        The list of NTP server IP addresses. Maximum of 3 allowed.
        """
        return pulumi.get(self, "ntps")

    @property
    @pulumi.getter
    def scans(self) -> Sequence['outputs.GetVmClusterNetworkScanResult']:
        """
        The SCAN details.
        """
        return pulumi.get(self, "scans")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        The current state of the VM cluster network nodes. CREATING - The resource is being created REQUIRES_VALIDATION - The resource is created and may not be usable until it is validated. VALIDATING - The resource is being validated and not available to use. VALIDATED - The resource is validated and is available for consumption by VM cluster. VALIDATION_FAILED - The resource validation has failed and might require user input to be corrected. UPDATING - The resource is being updated and not available to use. ALLOCATED - The resource is currently being used by VM cluster. TERMINATING - The resource is being deleted and not available to use. TERMINATED - The resource is deleted and unavailable. FAILED - The resource is in a failed state due to validation or other errors.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> str:
        """
        The date and time when the VM cluster network was created.
        """
        return pulumi.get(self, "time_created")

    @property
    @pulumi.getter(name="validateVmClusterNetwork")
    def validate_vm_cluster_network(self) -> bool:
        return pulumi.get(self, "validate_vm_cluster_network")

    @property
    @pulumi.getter(name="vmClusterId")
    def vm_cluster_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the associated VM Cluster.
        """
        return pulumi.get(self, "vm_cluster_id")

    @property
    @pulumi.getter(name="vmClusterNetworkId")
    def vm_cluster_network_id(self) -> str:
        return pulumi.get(self, "vm_cluster_network_id")

    @property
    @pulumi.getter(name="vmNetworks")
    def vm_networks(self) -> Sequence['outputs.GetVmClusterNetworkVmNetworkResult']:
        """
        Details of the client and backup networks.
        """
        return pulumi.get(self, "vm_networks")


class AwaitableGetVmClusterNetworkResult(GetVmClusterNetworkResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetVmClusterNetworkResult(
            action=self.action,
            compartment_id=self.compartment_id,
            defined_tags=self.defined_tags,
            display_name=self.display_name,
            dns=self.dns,
            exadata_infrastructure_id=self.exadata_infrastructure_id,
            freeform_tags=self.freeform_tags,
            id=self.id,
            lifecycle_details=self.lifecycle_details,
            ntps=self.ntps,
            scans=self.scans,
            state=self.state,
            time_created=self.time_created,
            validate_vm_cluster_network=self.validate_vm_cluster_network,
            vm_cluster_id=self.vm_cluster_id,
            vm_cluster_network_id=self.vm_cluster_network_id,
            vm_networks=self.vm_networks)


def get_vm_cluster_network(exadata_infrastructure_id: Optional[str] = None,
                           vm_cluster_network_id: Optional[str] = None,
                           opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetVmClusterNetworkResult:
    """
    This data source provides details about a specific Vm Cluster Network resource in Oracle Cloud Infrastructure Database service.

    Gets information about the specified VM cluster network. Applies to Exadata Cloud@Customer instances only.
    To get information about a cloud VM cluster in an Exadata Cloud Service instance, use the [GetCloudVmCluster ](https://docs.cloud.oracle.com/iaas/api/#/en/database/latest/CloudVmCluster/GetCloudVmCluster) operation.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_vm_cluster_network = oci.Database.get_vm_cluster_network(exadata_infrastructure_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        vm_cluster_network_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str exadata_infrastructure_id: The Exadata infrastructure [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    :param str vm_cluster_network_id: The VM cluster network [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    """
    __args__ = dict()
    __args__['exadataInfrastructureId'] = exadata_infrastructure_id
    __args__['vmClusterNetworkId'] = vm_cluster_network_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Database/getVmClusterNetwork:getVmClusterNetwork', __args__, opts=opts, typ=GetVmClusterNetworkResult).value

    return AwaitableGetVmClusterNetworkResult(
        action=__ret__.action,
        compartment_id=__ret__.compartment_id,
        defined_tags=__ret__.defined_tags,
        display_name=__ret__.display_name,
        dns=__ret__.dns,
        exadata_infrastructure_id=__ret__.exadata_infrastructure_id,
        freeform_tags=__ret__.freeform_tags,
        id=__ret__.id,
        lifecycle_details=__ret__.lifecycle_details,
        ntps=__ret__.ntps,
        scans=__ret__.scans,
        state=__ret__.state,
        time_created=__ret__.time_created,
        validate_vm_cluster_network=__ret__.validate_vm_cluster_network,
        vm_cluster_id=__ret__.vm_cluster_id,
        vm_cluster_network_id=__ret__.vm_cluster_network_id,
        vm_networks=__ret__.vm_networks)


@_utilities.lift_output_func(get_vm_cluster_network)
def get_vm_cluster_network_output(exadata_infrastructure_id: Optional[pulumi.Input[str]] = None,
                                  vm_cluster_network_id: Optional[pulumi.Input[str]] = None,
                                  opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetVmClusterNetworkResult]:
    """
    This data source provides details about a specific Vm Cluster Network resource in Oracle Cloud Infrastructure Database service.

    Gets information about the specified VM cluster network. Applies to Exadata Cloud@Customer instances only.
    To get information about a cloud VM cluster in an Exadata Cloud Service instance, use the [GetCloudVmCluster ](https://docs.cloud.oracle.com/iaas/api/#/en/database/latest/CloudVmCluster/GetCloudVmCluster) operation.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_vm_cluster_network = oci.Database.get_vm_cluster_network(exadata_infrastructure_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        vm_cluster_network_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str exadata_infrastructure_id: The Exadata infrastructure [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    :param str vm_cluster_network_id: The VM cluster network [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetAddressListResult',
    'AwaitableGetAddressListResult',
    'get_address_list',
    'get_address_list_output',
]

@pulumi.output_type
class GetAddressListResult:
    """
    A collection of values returned by getAddressList.
    """
    def __init__(__self__, address_count=None, address_list_id=None, addresses=None, compartment_id=None, defined_tags=None, display_name=None, freeform_tags=None, id=None, state=None, time_created=None):
        if address_count and not isinstance(address_count, float):
            raise TypeError("Expected argument 'address_count' to be a float")
        pulumi.set(__self__, "address_count", address_count)
        if address_list_id and not isinstance(address_list_id, str):
            raise TypeError("Expected argument 'address_list_id' to be a str")
        pulumi.set(__self__, "address_list_id", address_list_id)
        if addresses and not isinstance(addresses, list):
            raise TypeError("Expected argument 'addresses' to be a list")
        pulumi.set(__self__, "addresses", addresses)
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if defined_tags and not isinstance(defined_tags, dict):
            raise TypeError("Expected argument 'defined_tags' to be a dict")
        pulumi.set(__self__, "defined_tags", defined_tags)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if freeform_tags and not isinstance(freeform_tags, dict):
            raise TypeError("Expected argument 'freeform_tags' to be a dict")
        pulumi.set(__self__, "freeform_tags", freeform_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if time_created and not isinstance(time_created, str):
            raise TypeError("Expected argument 'time_created' to be a str")
        pulumi.set(__self__, "time_created", time_created)

    @property
    @pulumi.getter(name="addressCount")
    def address_count(self) -> float:
        """
        The total number of unique IP addresses in the address list.
        """
        return pulumi.get(self, "address_count")

    @property
    @pulumi.getter(name="addressListId")
    def address_list_id(self) -> str:
        return pulumi.get(self, "address_list_id")

    @property
    @pulumi.getter
    def addresses(self) -> Sequence[str]:
        """
        The list of IP addresses or CIDR notations.
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the address list's compartment.
        """
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter(name="definedTags")
    def defined_tags(self) -> Mapping[str, Any]:
        """
        Defined tags for this resource. Each key is predefined and scoped to a namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Operations.CostCenter": "42"}`
        """
        return pulumi.get(self, "defined_tags")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        The user-friendly name of the address list.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="freeformTags")
    def freeform_tags(self) -> Mapping[str, Any]:
        """
        Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm).  Example: `{"Department": "Finance"}`
        """
        return pulumi.get(self, "freeform_tags")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the address list.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def state(self) -> str:
        """
        The current lifecycle state of the address list.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="timeCreated")
    def time_created(self) -> str:
        """
        The date and time the address list was created, expressed in RFC 3339 timestamp format.
        """
        return pulumi.get(self, "time_created")


class AwaitableGetAddressListResult(GetAddressListResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAddressListResult(
            address_count=self.address_count,
            address_list_id=self.address_list_id,
            addresses=self.addresses,
            compartment_id=self.compartment_id,
            defined_tags=self.defined_tags,
            display_name=self.display_name,
            freeform_tags=self.freeform_tags,
            id=self.id,
            state=self.state,
            time_created=self.time_created)


def get_address_list(address_list_id: Optional[str] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAddressListResult:
    """
    This data source provides details about a specific Address List resource in Oracle Cloud Infrastructure Web Application Acceleration and Security service.

    Gets the details of an address list.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_address_list = oci.Waas.get_address_list(address_list_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str address_list_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the address list. This number is generated when the address list is added to the compartment.
    """
    __args__ = dict()
    __args__['addressListId'] = address_list_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Waas/getAddressList:getAddressList', __args__, opts=opts, typ=GetAddressListResult).value

    return AwaitableGetAddressListResult(
        address_count=__ret__.address_count,
        address_list_id=__ret__.address_list_id,
        addresses=__ret__.addresses,
        compartment_id=__ret__.compartment_id,
        defined_tags=__ret__.defined_tags,
        display_name=__ret__.display_name,
        freeform_tags=__ret__.freeform_tags,
        id=__ret__.id,
        state=__ret__.state,
        time_created=__ret__.time_created)


@_utilities.lift_output_func(get_address_list)
def get_address_list_output(address_list_id: Optional[pulumi.Input[str]] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetAddressListResult]:
    """
    This data source provides details about a specific Address List resource in Oracle Cloud Infrastructure Web Application Acceleration and Security service.

    Gets the details of an address list.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_address_list = oci.Waas.get_address_list(address_list_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str address_list_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the address list. This number is generated when the address list is added to the compartment.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetDomainsUserResult',
    'AwaitableGetDomainsUserResult',
    'get_domains_user',
    'get_domains_user_output',
]

@pulumi.output_type
class GetDomainsUserResult:
    """
    A collection of values returned by getDomainsUser.
    """
    def __init__(__self__, active=None, addresses=None, attribute_sets=None, attributes=None, authorization=None, compartment_ocid=None, delete_in_progress=None, description=None, display_name=None, domain_ocid=None, emails=None, entitlements=None, external_id=None, groups=None, id=None, idcs_created_bies=None, idcs_endpoint=None, idcs_last_modified_bies=None, idcs_last_upgraded_in_release=None, idcs_prevented_operations=None, ims=None, locale=None, metas=None, names=None, nick_name=None, ocid=None, password=None, phone_numbers=None, photos=None, preferred_language=None, profile_url=None, resource_type_schema_version=None, roles=None, schemas=None, tags=None, tenancy_ocid=None, timezone=None, title=None, urnietfparamsscimschemasextensionenterprise20users=None, urnietfparamsscimschemasoracleidcsextension_oci_tags=None, urnietfparamsscimschemasoracleidcsextensionadaptive_users=None, urnietfparamsscimschemasoracleidcsextensioncapabilities_users=None, urnietfparamsscimschemasoracleidcsextensiondb_credentials_users=None, urnietfparamsscimschemasoracleidcsextensiondb_user_users=None, urnietfparamsscimschemasoracleidcsextensionkerberos_user_users=None, urnietfparamsscimschemasoracleidcsextensionmfa_users=None, urnietfparamsscimschemasoracleidcsextensionpassword_state_users=None, urnietfparamsscimschemasoracleidcsextensionpasswordless_users=None, urnietfparamsscimschemasoracleidcsextensionposix_users=None, urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users=None, urnietfparamsscimschemasoracleidcsextensionself_change_users=None, urnietfparamsscimschemasoracleidcsextensionself_registration_users=None, urnietfparamsscimschemasoracleidcsextensionsff_users=None, urnietfparamsscimschemasoracleidcsextensionsocial_account_users=None, urnietfparamsscimschemasoracleidcsextensionterms_of_use_users=None, urnietfparamsscimschemasoracleidcsextensionuser_credentials_users=None, urnietfparamsscimschemasoracleidcsextensionuser_state_users=None, urnietfparamsscimschemasoracleidcsextensionuser_users=None, user_id=None, user_name=None, user_type=None, x509certificates=None):
        if active and not isinstance(active, bool):
            raise TypeError("Expected argument 'active' to be a bool")
        pulumi.set(__self__, "active", active)
        if addresses and not isinstance(addresses, list):
            raise TypeError("Expected argument 'addresses' to be a list")
        pulumi.set(__self__, "addresses", addresses)
        if attribute_sets and not isinstance(attribute_sets, list):
            raise TypeError("Expected argument 'attribute_sets' to be a list")
        pulumi.set(__self__, "attribute_sets", attribute_sets)
        if attributes and not isinstance(attributes, str):
            raise TypeError("Expected argument 'attributes' to be a str")
        pulumi.set(__self__, "attributes", attributes)
        if authorization and not isinstance(authorization, str):
            raise TypeError("Expected argument 'authorization' to be a str")
        pulumi.set(__self__, "authorization", authorization)
        if compartment_ocid and not isinstance(compartment_ocid, str):
            raise TypeError("Expected argument 'compartment_ocid' to be a str")
        pulumi.set(__self__, "compartment_ocid", compartment_ocid)
        if delete_in_progress and not isinstance(delete_in_progress, bool):
            raise TypeError("Expected argument 'delete_in_progress' to be a bool")
        pulumi.set(__self__, "delete_in_progress", delete_in_progress)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if domain_ocid and not isinstance(domain_ocid, str):
            raise TypeError("Expected argument 'domain_ocid' to be a str")
        pulumi.set(__self__, "domain_ocid", domain_ocid)
        if emails and not isinstance(emails, list):
            raise TypeError("Expected argument 'emails' to be a list")
        pulumi.set(__self__, "emails", emails)
        if entitlements and not isinstance(entitlements, list):
            raise TypeError("Expected argument 'entitlements' to be a list")
        pulumi.set(__self__, "entitlements", entitlements)
        if external_id and not isinstance(external_id, str):
            raise TypeError("Expected argument 'external_id' to be a str")
        pulumi.set(__self__, "external_id", external_id)
        if groups and not isinstance(groups, list):
            raise TypeError("Expected argument 'groups' to be a list")
        pulumi.set(__self__, "groups", groups)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if idcs_created_bies and not isinstance(idcs_created_bies, list):
            raise TypeError("Expected argument 'idcs_created_bies' to be a list")
        pulumi.set(__self__, "idcs_created_bies", idcs_created_bies)
        if idcs_endpoint and not isinstance(idcs_endpoint, str):
            raise TypeError("Expected argument 'idcs_endpoint' to be a str")
        pulumi.set(__self__, "idcs_endpoint", idcs_endpoint)
        if idcs_last_modified_bies and not isinstance(idcs_last_modified_bies, list):
            raise TypeError("Expected argument 'idcs_last_modified_bies' to be a list")
        pulumi.set(__self__, "idcs_last_modified_bies", idcs_last_modified_bies)
        if idcs_last_upgraded_in_release and not isinstance(idcs_last_upgraded_in_release, str):
            raise TypeError("Expected argument 'idcs_last_upgraded_in_release' to be a str")
        pulumi.set(__self__, "idcs_last_upgraded_in_release", idcs_last_upgraded_in_release)
        if idcs_prevented_operations and not isinstance(idcs_prevented_operations, list):
            raise TypeError("Expected argument 'idcs_prevented_operations' to be a list")
        pulumi.set(__self__, "idcs_prevented_operations", idcs_prevented_operations)
        if ims and not isinstance(ims, list):
            raise TypeError("Expected argument 'ims' to be a list")
        pulumi.set(__self__, "ims", ims)
        if locale and not isinstance(locale, str):
            raise TypeError("Expected argument 'locale' to be a str")
        pulumi.set(__self__, "locale", locale)
        if metas and not isinstance(metas, list):
            raise TypeError("Expected argument 'metas' to be a list")
        pulumi.set(__self__, "metas", metas)
        if names and not isinstance(names, list):
            raise TypeError("Expected argument 'names' to be a list")
        pulumi.set(__self__, "names", names)
        if nick_name and not isinstance(nick_name, str):
            raise TypeError("Expected argument 'nick_name' to be a str")
        pulumi.set(__self__, "nick_name", nick_name)
        if ocid and not isinstance(ocid, str):
            raise TypeError("Expected argument 'ocid' to be a str")
        pulumi.set(__self__, "ocid", ocid)
        if password and not isinstance(password, str):
            raise TypeError("Expected argument 'password' to be a str")
        pulumi.set(__self__, "password", password)
        if phone_numbers and not isinstance(phone_numbers, list):
            raise TypeError("Expected argument 'phone_numbers' to be a list")
        pulumi.set(__self__, "phone_numbers", phone_numbers)
        if photos and not isinstance(photos, list):
            raise TypeError("Expected argument 'photos' to be a list")
        pulumi.set(__self__, "photos", photos)
        if preferred_language and not isinstance(preferred_language, str):
            raise TypeError("Expected argument 'preferred_language' to be a str")
        pulumi.set(__self__, "preferred_language", preferred_language)
        if profile_url and not isinstance(profile_url, str):
            raise TypeError("Expected argument 'profile_url' to be a str")
        pulumi.set(__self__, "profile_url", profile_url)
        if resource_type_schema_version and not isinstance(resource_type_schema_version, str):
            raise TypeError("Expected argument 'resource_type_schema_version' to be a str")
        pulumi.set(__self__, "resource_type_schema_version", resource_type_schema_version)
        if roles and not isinstance(roles, list):
            raise TypeError("Expected argument 'roles' to be a list")
        pulumi.set(__self__, "roles", roles)
        if schemas and not isinstance(schemas, list):
            raise TypeError("Expected argument 'schemas' to be a list")
        pulumi.set(__self__, "schemas", schemas)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if tenancy_ocid and not isinstance(tenancy_ocid, str):
            raise TypeError("Expected argument 'tenancy_ocid' to be a str")
        pulumi.set(__self__, "tenancy_ocid", tenancy_ocid)
        if timezone and not isinstance(timezone, str):
            raise TypeError("Expected argument 'timezone' to be a str")
        pulumi.set(__self__, "timezone", timezone)
        if title and not isinstance(title, str):
            raise TypeError("Expected argument 'title' to be a str")
        pulumi.set(__self__, "title", title)
        if urnietfparamsscimschemasextensionenterprise20users and not isinstance(urnietfparamsscimschemasextensionenterprise20users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasextensionenterprise20users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasextensionenterprise20users", urnietfparamsscimschemasextensionenterprise20users)
        if urnietfparamsscimschemasoracleidcsextension_oci_tags and not isinstance(urnietfparamsscimschemasoracleidcsextension_oci_tags, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextension_oci_tags' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextension_oci_tags", urnietfparamsscimschemasoracleidcsextension_oci_tags)
        if urnietfparamsscimschemasoracleidcsextensionadaptive_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionadaptive_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionadaptive_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionadaptive_users", urnietfparamsscimschemasoracleidcsextensionadaptive_users)
        if urnietfparamsscimschemasoracleidcsextensioncapabilities_users and not isinstance(urnietfparamsscimschemasoracleidcsextensioncapabilities_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensioncapabilities_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensioncapabilities_users", urnietfparamsscimschemasoracleidcsextensioncapabilities_users)
        if urnietfparamsscimschemasoracleidcsextensiondb_credentials_users and not isinstance(urnietfparamsscimschemasoracleidcsextensiondb_credentials_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensiondb_credentials_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensiondb_credentials_users", urnietfparamsscimschemasoracleidcsextensiondb_credentials_users)
        if urnietfparamsscimschemasoracleidcsextensiondb_user_users and not isinstance(urnietfparamsscimschemasoracleidcsextensiondb_user_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensiondb_user_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensiondb_user_users", urnietfparamsscimschemasoracleidcsextensiondb_user_users)
        if urnietfparamsscimschemasoracleidcsextensionkerberos_user_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionkerberos_user_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionkerberos_user_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionkerberos_user_users", urnietfparamsscimschemasoracleidcsextensionkerberos_user_users)
        if urnietfparamsscimschemasoracleidcsextensionmfa_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionmfa_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionmfa_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionmfa_users", urnietfparamsscimschemasoracleidcsextensionmfa_users)
        if urnietfparamsscimschemasoracleidcsextensionpassword_state_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionpassword_state_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionpassword_state_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionpassword_state_users", urnietfparamsscimschemasoracleidcsextensionpassword_state_users)
        if urnietfparamsscimschemasoracleidcsextensionpasswordless_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionpasswordless_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionpasswordless_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionpasswordless_users", urnietfparamsscimschemasoracleidcsextensionpasswordless_users)
        if urnietfparamsscimschemasoracleidcsextensionposix_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionposix_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionposix_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionposix_users", urnietfparamsscimschemasoracleidcsextensionposix_users)
        if urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users", urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users)
        if urnietfparamsscimschemasoracleidcsextensionself_change_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionself_change_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionself_change_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionself_change_users", urnietfparamsscimschemasoracleidcsextensionself_change_users)
        if urnietfparamsscimschemasoracleidcsextensionself_registration_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionself_registration_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionself_registration_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionself_registration_users", urnietfparamsscimschemasoracleidcsextensionself_registration_users)
        if urnietfparamsscimschemasoracleidcsextensionsff_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionsff_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionsff_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionsff_users", urnietfparamsscimschemasoracleidcsextensionsff_users)
        if urnietfparamsscimschemasoracleidcsextensionsocial_account_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionsocial_account_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionsocial_account_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionsocial_account_users", urnietfparamsscimschemasoracleidcsextensionsocial_account_users)
        if urnietfparamsscimschemasoracleidcsextensionterms_of_use_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionterms_of_use_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionterms_of_use_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionterms_of_use_users", urnietfparamsscimschemasoracleidcsextensionterms_of_use_users)
        if urnietfparamsscimschemasoracleidcsextensionuser_credentials_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionuser_credentials_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionuser_credentials_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionuser_credentials_users", urnietfparamsscimschemasoracleidcsextensionuser_credentials_users)
        if urnietfparamsscimschemasoracleidcsextensionuser_state_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionuser_state_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionuser_state_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionuser_state_users", urnietfparamsscimschemasoracleidcsextensionuser_state_users)
        if urnietfparamsscimschemasoracleidcsextensionuser_users and not isinstance(urnietfparamsscimschemasoracleidcsextensionuser_users, list):
            raise TypeError("Expected argument 'urnietfparamsscimschemasoracleidcsextensionuser_users' to be a list")
        pulumi.set(__self__, "urnietfparamsscimschemasoracleidcsextensionuser_users", urnietfparamsscimschemasoracleidcsextensionuser_users)
        if user_id and not isinstance(user_id, str):
            raise TypeError("Expected argument 'user_id' to be a str")
        pulumi.set(__self__, "user_id", user_id)
        if user_name and not isinstance(user_name, str):
            raise TypeError("Expected argument 'user_name' to be a str")
        pulumi.set(__self__, "user_name", user_name)
        if user_type and not isinstance(user_type, str):
            raise TypeError("Expected argument 'user_type' to be a str")
        pulumi.set(__self__, "user_type", user_type)
        if x509certificates and not isinstance(x509certificates, list):
            raise TypeError("Expected argument 'x509certificates' to be a list")
        pulumi.set(__self__, "x509certificates", x509certificates)

    @property
    @pulumi.getter
    def active(self) -> bool:
        """
        Status of the account
        """
        return pulumi.get(self, "active")

    @property
    @pulumi.getter
    def addresses(self) -> Sequence['outputs.GetDomainsUserAddressResult']:
        """
        A physical mailing address for this User, as described in (address Element). Canonical Type Values of work, home, and other. The value attribute is a complex type with the following sub-attributes.
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter(name="attributeSets")
    def attribute_sets(self) -> Optional[Sequence[str]]:
        return pulumi.get(self, "attribute_sets")

    @property
    @pulumi.getter
    def attributes(self) -> Optional[str]:
        return pulumi.get(self, "attributes")

    @property
    @pulumi.getter
    def authorization(self) -> Optional[str]:
        return pulumi.get(self, "authorization")

    @property
    @pulumi.getter(name="compartmentOcid")
    def compartment_ocid(self) -> str:
        """
        Oracle Cloud Infrastructure Compartment Id (ocid) in which the resource lives.
        """
        return pulumi.get(self, "compartment_ocid")

    @property
    @pulumi.getter(name="deleteInProgress")
    def delete_in_progress(self) -> bool:
        """
        A boolean flag indicating this resource in the process of being deleted. Usually set to true when synchronous deletion of the resource would take too long.
        """
        return pulumi.get(self, "delete_in_progress")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        Description of the user
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        The displayName of the User's manager. OPTIONAL and READ-ONLY.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="domainOcid")
    def domain_ocid(self) -> str:
        """
        Oracle Cloud Infrastructure Domain Id (ocid) in which the resource lives.
        """
        return pulumi.get(self, "domain_ocid")

    @property
    @pulumi.getter
    def emails(self) -> Sequence['outputs.GetDomainsUserEmailResult']:
        """
        A complex attribute representing emails
        """
        return pulumi.get(self, "emails")

    @property
    @pulumi.getter
    def entitlements(self) -> Sequence['outputs.GetDomainsUserEntitlementResult']:
        """
        A list of entitlements for the User that represent a thing the User has.
        """
        return pulumi.get(self, "entitlements")

    @property
    @pulumi.getter(name="externalId")
    def external_id(self) -> str:
        """
        An identifier for the Resource as defined by the Service Consumer. READ-ONLY.
        """
        return pulumi.get(self, "external_id")

    @property
    @pulumi.getter
    def groups(self) -> Sequence['outputs.GetDomainsUserGroupResult']:
        """
        A list of groups that the user belongs to, either thorough direct membership, nested groups, or dynamically calculated
        """
        return pulumi.get(self, "groups")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        Unique identifier for the SCIM Resource as defined by the Service Provider. Each representation of the Resource MUST include a non-empty id value. This identifier MUST be unique across the Service Provider's entire set of Resources. It MUST be a stable, non-reassignable identifier that does not change when the same Resource is returned in subsequent requests. The value of the id attribute is always issued by the Service Provider and MUST never be specified by the Service Consumer. bulkId: is a reserved keyword and MUST NOT be used in the unique identifier.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="idcsCreatedBies")
    def idcs_created_bies(self) -> Sequence['outputs.GetDomainsUserIdcsCreatedByResult']:
        """
        The User or App who created the Resource
        """
        return pulumi.get(self, "idcs_created_bies")

    @property
    @pulumi.getter(name="idcsEndpoint")
    def idcs_endpoint(self) -> str:
        return pulumi.get(self, "idcs_endpoint")

    @property
    @pulumi.getter(name="idcsLastModifiedBies")
    def idcs_last_modified_bies(self) -> Sequence['outputs.GetDomainsUserIdcsLastModifiedByResult']:
        """
        The User or App who modified the Resource
        """
        return pulumi.get(self, "idcs_last_modified_bies")

    @property
    @pulumi.getter(name="idcsLastUpgradedInRelease")
    def idcs_last_upgraded_in_release(self) -> str:
        """
        The release number when the resource was upgraded.
        """
        return pulumi.get(self, "idcs_last_upgraded_in_release")

    @property
    @pulumi.getter(name="idcsPreventedOperations")
    def idcs_prevented_operations(self) -> Sequence[str]:
        """
        Each value of this attribute specifies an operation that only an internal client may perform on this particular resource.
        """
        return pulumi.get(self, "idcs_prevented_operations")

    @property
    @pulumi.getter
    def ims(self) -> Sequence['outputs.GetDomainsUserImResult']:
        """
        User's instant messaging addresses
        """
        return pulumi.get(self, "ims")

    @property
    @pulumi.getter
    def locale(self) -> str:
        """
        Used to indicate the User's default location for purposes of localizing items such as currency, date and time format, numerical representations, and so on.
        """
        return pulumi.get(self, "locale")

    @property
    @pulumi.getter
    def metas(self) -> Sequence['outputs.GetDomainsUserMetaResult']:
        """
        A complex attribute that contains resource metadata. All sub-attributes are OPTIONAL.
        """
        return pulumi.get(self, "metas")

    @property
    @pulumi.getter
    def names(self) -> Sequence['outputs.GetDomainsUserNameResult']:
        """
        Name of the account assigned to the User.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="nickName")
    def nick_name(self) -> str:
        """
        Nick name
        """
        return pulumi.get(self, "nick_name")

    @property
    @pulumi.getter
    def ocid(self) -> str:
        """
        Ocid of the User's Support Account.
        """
        return pulumi.get(self, "ocid")

    @property
    @pulumi.getter
    def password(self) -> str:
        """
        Password attribute. Max length for password is controlled via Password Policy.
        """
        return pulumi.get(self, "password")

    @property
    @pulumi.getter(name="phoneNumbers")
    def phone_numbers(self) -> Sequence['outputs.GetDomainsUserPhoneNumberResult']:
        """
        Phone numbers
        """
        return pulumi.get(self, "phone_numbers")

    @property
    @pulumi.getter
    def photos(self) -> Sequence['outputs.GetDomainsUserPhotoResult']:
        """
        URLs of photos for the User
        """
        return pulumi.get(self, "photos")

    @property
    @pulumi.getter(name="preferredLanguage")
    def preferred_language(self) -> str:
        """
        User's preferred written or spoken language used for localized user interfaces
        """
        return pulumi.get(self, "preferred_language")

    @property
    @pulumi.getter(name="profileUrl")
    def profile_url(self) -> str:
        """
        A fully-qualified URL to a page representing the User's online profile
        """
        return pulumi.get(self, "profile_url")

    @property
    @pulumi.getter(name="resourceTypeSchemaVersion")
    def resource_type_schema_version(self) -> Optional[str]:
        return pulumi.get(self, "resource_type_schema_version")

    @property
    @pulumi.getter
    def roles(self) -> Sequence['outputs.GetDomainsUserRoleResult']:
        """
        A list of roles for the User that collectively represent who the User is; e.g., 'Student', 'Faculty'.
        """
        return pulumi.get(self, "roles")

    @property
    @pulumi.getter
    def schemas(self) -> Sequence[str]:
        """
        REQUIRED. The schemas attribute is an array of Strings which allows introspection of the supported schema version for a SCIM representation as well any schema extensions supported by that representation. Each String value must be a unique URI. This specification defines URIs for User, Group, and a standard \\"enterprise\\" extension. All representations of SCIM schema MUST include a non-zero value array with value(s) of the URIs supported by that representation. Duplicate values MUST NOT be included. Value order is not specified and MUST not impact behavior.
        """
        return pulumi.get(self, "schemas")

    @property
    @pulumi.getter
    def tags(self) -> Sequence['outputs.GetDomainsUserTagResult']:
        """
        A list of tags on this resource.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter(name="tenancyOcid")
    def tenancy_ocid(self) -> str:
        """
        Oracle Cloud Infrastructure Tenant Id (ocid) in which the resource lives.
        """
        return pulumi.get(self, "tenancy_ocid")

    @property
    @pulumi.getter
    def timezone(self) -> str:
        """
        User's timezone
        """
        return pulumi.get(self, "timezone")

    @property
    @pulumi.getter
    def title(self) -> str:
        """
        Title
        """
        return pulumi.get(self, "title")

    @property
    @pulumi.getter
    def urnietfparamsscimschemasextensionenterprise20users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasextensionenterprise20userResult']:
        """
        Enterprise User
        """
        return pulumi.get(self, "urnietfparamsscimschemasextensionenterprise20users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionOciTags")
    def urnietfparamsscimschemasoracleidcsextension_oci_tags(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionOciTagResult']:
        """
        Oracle Cloud Infrastructure Tags.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextension_oci_tags")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionadaptiveUsers")
    def urnietfparamsscimschemasoracleidcsextensionadaptive_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionadaptiveUserResult']:
        """
        This extension defines attributes to manage user's risk score.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionadaptive_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensioncapabilitiesUsers")
    def urnietfparamsscimschemasoracleidcsextensioncapabilities_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensioncapabilitiesUserResult']:
        """
        User's Capabilities
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensioncapabilities_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensiondbCredentialsUsers")
    def urnietfparamsscimschemasoracleidcsextensiondb_credentials_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensiondbCredentialsUserResult']:
        """
        Db Credentials User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensiondb_credentials_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensiondbUserUsers")
    def urnietfparamsscimschemasoracleidcsextensiondb_user_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensiondbUserUserResult']:
        """
        DB User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensiondb_user_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionkerberosUserUsers")
    def urnietfparamsscimschemasoracleidcsextensionkerberos_user_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionkerberosUserUserResult']:
        """
        Kerberos User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionkerberos_user_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionmfaUsers")
    def urnietfparamsscimschemasoracleidcsextensionmfa_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionmfaUserResult']:
        """
        This extension defines attributes used to manage Multi-Factor Authentication within a service provider. The extension is typically applied to a User resource, but MAY be applied to other resources that use MFA.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionmfa_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionpasswordStateUsers")
    def urnietfparamsscimschemasoracleidcsextensionpassword_state_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionpasswordStateUserResult']:
        """
        This extension defines attributes used to manage account passwords within a Service Provider. The extension is typically applied to a User resource, but MAY be applied to other resources that use passwords.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionpassword_state_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionpasswordlessUsers")
    def urnietfparamsscimschemasoracleidcsextensionpasswordless_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionpasswordlessUserResult']:
        """
        This extension defines attributes used to manage Passwordless-Factor Authentication within a service provider. The extension is typically applied to a User resource, but MAY be applied to other resources that use MFA.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionpasswordless_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionposixUsers")
    def urnietfparamsscimschemasoracleidcsextensionposix_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionposixUserResult']:
        """
        POSIX User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionposix_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionsecurityQuestionsUsers")
    def urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionsecurityQuestionsUserResult']:
        """
        This extension defines attributes used to store Security Questions of User.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionselfChangeUsers")
    def urnietfparamsscimschemasoracleidcsextensionself_change_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionselfChangeUserResult']:
        """
        Controls whether a user can update themselves or not via User related APIs
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionself_change_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionselfRegistrationUsers")
    def urnietfparamsscimschemasoracleidcsextensionself_registration_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionselfRegistrationUserResult']:
        """
        This extension defines attributes used to manage self registration profile linked to the user.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionself_registration_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionsffUsers")
    def urnietfparamsscimschemasoracleidcsextensionsff_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionsffUserResult']:
        """
        SFF Auth Keys User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionsff_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionsocialAccountUsers")
    def urnietfparamsscimschemasoracleidcsextensionsocial_account_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionsocialAccountUserResult']:
        """
        Social User extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionsocial_account_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensiontermsOfUseUsers")
    def urnietfparamsscimschemasoracleidcsextensionterms_of_use_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensiontermsOfUseUserResult']:
        """
        Terms Of Use extension
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionterms_of_use_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionuserCredentialsUsers")
    def urnietfparamsscimschemasoracleidcsextensionuser_credentials_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionuserCredentialsUserResult']:
        """
        User's credentials
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionuser_credentials_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionuserStateUsers")
    def urnietfparamsscimschemasoracleidcsextensionuser_state_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionuserStateUserResult']:
        """
        This extension defines attributes used to manage account passwords within a service provider. The extension is typically applied to a User resource, but MAY be applied to other resources that use passwords.
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionuser_state_users")

    @property
    @pulumi.getter(name="urnietfparamsscimschemasoracleidcsextensionuserUsers")
    def urnietfparamsscimschemasoracleidcsextensionuser_users(self) -> Sequence['outputs.GetDomainsUserUrnietfparamsscimschemasoracleidcsextensionuserUserResult']:
        """
        Oracle Cloud Infrastructure IAM User
        """
        return pulumi.get(self, "urnietfparamsscimschemasoracleidcsextensionuser_users")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> str:
        """
        User Support User Id
        """
        return pulumi.get(self, "user_id")

    @property
    @pulumi.getter(name="userName")
    def user_name(self) -> str:
        """
        User name
        """
        return pulumi.get(self, "user_name")

    @property
    @pulumi.getter(name="userType")
    def user_type(self) -> str:
        """
        Used to identify the organization-to-user relationship
        """
        return pulumi.get(self, "user_type")

    @property
    @pulumi.getter
    def x509certificates(self) -> Sequence['outputs.GetDomainsUserX509certificateResult']:
        """
        A list of certificates issued to the User.
        """
        return pulumi.get(self, "x509certificates")


class AwaitableGetDomainsUserResult(GetDomainsUserResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDomainsUserResult(
            active=self.active,
            addresses=self.addresses,
            attribute_sets=self.attribute_sets,
            attributes=self.attributes,
            authorization=self.authorization,
            compartment_ocid=self.compartment_ocid,
            delete_in_progress=self.delete_in_progress,
            description=self.description,
            display_name=self.display_name,
            domain_ocid=self.domain_ocid,
            emails=self.emails,
            entitlements=self.entitlements,
            external_id=self.external_id,
            groups=self.groups,
            id=self.id,
            idcs_created_bies=self.idcs_created_bies,
            idcs_endpoint=self.idcs_endpoint,
            idcs_last_modified_bies=self.idcs_last_modified_bies,
            idcs_last_upgraded_in_release=self.idcs_last_upgraded_in_release,
            idcs_prevented_operations=self.idcs_prevented_operations,
            ims=self.ims,
            locale=self.locale,
            metas=self.metas,
            names=self.names,
            nick_name=self.nick_name,
            ocid=self.ocid,
            password=self.password,
            phone_numbers=self.phone_numbers,
            photos=self.photos,
            preferred_language=self.preferred_language,
            profile_url=self.profile_url,
            resource_type_schema_version=self.resource_type_schema_version,
            roles=self.roles,
            schemas=self.schemas,
            tags=self.tags,
            tenancy_ocid=self.tenancy_ocid,
            timezone=self.timezone,
            title=self.title,
            urnietfparamsscimschemasextensionenterprise20users=self.urnietfparamsscimschemasextensionenterprise20users,
            urnietfparamsscimschemasoracleidcsextension_oci_tags=self.urnietfparamsscimschemasoracleidcsextension_oci_tags,
            urnietfparamsscimschemasoracleidcsextensionadaptive_users=self.urnietfparamsscimschemasoracleidcsextensionadaptive_users,
            urnietfparamsscimschemasoracleidcsextensioncapabilities_users=self.urnietfparamsscimschemasoracleidcsextensioncapabilities_users,
            urnietfparamsscimschemasoracleidcsextensiondb_credentials_users=self.urnietfparamsscimschemasoracleidcsextensiondb_credentials_users,
            urnietfparamsscimschemasoracleidcsextensiondb_user_users=self.urnietfparamsscimschemasoracleidcsextensiondb_user_users,
            urnietfparamsscimschemasoracleidcsextensionkerberos_user_users=self.urnietfparamsscimschemasoracleidcsextensionkerberos_user_users,
            urnietfparamsscimschemasoracleidcsextensionmfa_users=self.urnietfparamsscimschemasoracleidcsextensionmfa_users,
            urnietfparamsscimschemasoracleidcsextensionpassword_state_users=self.urnietfparamsscimschemasoracleidcsextensionpassword_state_users,
            urnietfparamsscimschemasoracleidcsextensionpasswordless_users=self.urnietfparamsscimschemasoracleidcsextensionpasswordless_users,
            urnietfparamsscimschemasoracleidcsextensionposix_users=self.urnietfparamsscimschemasoracleidcsextensionposix_users,
            urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users=self.urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users,
            urnietfparamsscimschemasoracleidcsextensionself_change_users=self.urnietfparamsscimschemasoracleidcsextensionself_change_users,
            urnietfparamsscimschemasoracleidcsextensionself_registration_users=self.urnietfparamsscimschemasoracleidcsextensionself_registration_users,
            urnietfparamsscimschemasoracleidcsextensionsff_users=self.urnietfparamsscimschemasoracleidcsextensionsff_users,
            urnietfparamsscimschemasoracleidcsextensionsocial_account_users=self.urnietfparamsscimschemasoracleidcsextensionsocial_account_users,
            urnietfparamsscimschemasoracleidcsextensionterms_of_use_users=self.urnietfparamsscimschemasoracleidcsextensionterms_of_use_users,
            urnietfparamsscimschemasoracleidcsextensionuser_credentials_users=self.urnietfparamsscimschemasoracleidcsextensionuser_credentials_users,
            urnietfparamsscimschemasoracleidcsextensionuser_state_users=self.urnietfparamsscimschemasoracleidcsextensionuser_state_users,
            urnietfparamsscimschemasoracleidcsextensionuser_users=self.urnietfparamsscimschemasoracleidcsextensionuser_users,
            user_id=self.user_id,
            user_name=self.user_name,
            user_type=self.user_type,
            x509certificates=self.x509certificates)


def get_domains_user(attribute_sets: Optional[Sequence[str]] = None,
                     attributes: Optional[str] = None,
                     authorization: Optional[str] = None,
                     idcs_endpoint: Optional[str] = None,
                     resource_type_schema_version: Optional[str] = None,
                     user_id: Optional[str] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDomainsUserResult:
    """
    This data source provides details about a specific User resource in Oracle Cloud Infrastructure Identity Domains service.

    Get a User

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_user = oci.Identity.get_domains_user(idcs_endpoint=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        user_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        attribute_sets=[],
        attributes="",
        authorization=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        resource_type_schema_version=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param Sequence[str] attribute_sets: A multi-valued list of strings indicating the return type of attribute definition. The specified set of attributes can be fetched by the return type of the attribute. One or more values can be given together to fetch more than one group of attributes. If 'attributes' query parameter is also available, union of the two is fetched. Valid values - all, always, never, request, default. Values are case-insensitive.
    :param str attributes: A comma-delimited string that specifies the names of resource attributes that should be returned in the response. By default, a response that contains resource attributes contains only attributes that are defined in the schema for that resource type as returned=always or returned=default. An attribute that is defined as returned=request is returned in a response only if the request specifies its name in the value of this query parameter. If a request specifies this query parameter, the response contains the attributes that this query parameter specifies, as well as any attribute that is defined as returned=always.
    :param str authorization: The Authorization field value consists of credentials containing the authentication information of the user agent for the realm of the resource being requested.
    :param str idcs_endpoint: The basic endpoint for the identity domain
    :param str resource_type_schema_version: An endpoint-specific schema version number to use in the Request. Allowed version values are Earliest Version or Latest Version as specified in each REST API endpoint description, or any sequential number inbetween. All schema attributes/body parameters are a part of version 1. After version 1, any attributes added or deprecated will be tagged with the version that they were added to or deprecated in. If no version is provided, the latest schema version is returned.
    :param str user_id: ID of the resource
    """
    __args__ = dict()
    __args__['attributeSets'] = attribute_sets
    __args__['attributes'] = attributes
    __args__['authorization'] = authorization
    __args__['idcsEndpoint'] = idcs_endpoint
    __args__['resourceTypeSchemaVersion'] = resource_type_schema_version
    __args__['userId'] = user_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:Identity/getDomainsUser:getDomainsUser', __args__, opts=opts, typ=GetDomainsUserResult).value

    return AwaitableGetDomainsUserResult(
        active=__ret__.active,
        addresses=__ret__.addresses,
        attribute_sets=__ret__.attribute_sets,
        attributes=__ret__.attributes,
        authorization=__ret__.authorization,
        compartment_ocid=__ret__.compartment_ocid,
        delete_in_progress=__ret__.delete_in_progress,
        description=__ret__.description,
        display_name=__ret__.display_name,
        domain_ocid=__ret__.domain_ocid,
        emails=__ret__.emails,
        entitlements=__ret__.entitlements,
        external_id=__ret__.external_id,
        groups=__ret__.groups,
        id=__ret__.id,
        idcs_created_bies=__ret__.idcs_created_bies,
        idcs_endpoint=__ret__.idcs_endpoint,
        idcs_last_modified_bies=__ret__.idcs_last_modified_bies,
        idcs_last_upgraded_in_release=__ret__.idcs_last_upgraded_in_release,
        idcs_prevented_operations=__ret__.idcs_prevented_operations,
        ims=__ret__.ims,
        locale=__ret__.locale,
        metas=__ret__.metas,
        names=__ret__.names,
        nick_name=__ret__.nick_name,
        ocid=__ret__.ocid,
        password=__ret__.password,
        phone_numbers=__ret__.phone_numbers,
        photos=__ret__.photos,
        preferred_language=__ret__.preferred_language,
        profile_url=__ret__.profile_url,
        resource_type_schema_version=__ret__.resource_type_schema_version,
        roles=__ret__.roles,
        schemas=__ret__.schemas,
        tags=__ret__.tags,
        tenancy_ocid=__ret__.tenancy_ocid,
        timezone=__ret__.timezone,
        title=__ret__.title,
        urnietfparamsscimschemasextensionenterprise20users=__ret__.urnietfparamsscimschemasextensionenterprise20users,
        urnietfparamsscimschemasoracleidcsextension_oci_tags=__ret__.urnietfparamsscimschemasoracleidcsextension_oci_tags,
        urnietfparamsscimschemasoracleidcsextensionadaptive_users=__ret__.urnietfparamsscimschemasoracleidcsextensionadaptive_users,
        urnietfparamsscimschemasoracleidcsextensioncapabilities_users=__ret__.urnietfparamsscimschemasoracleidcsextensioncapabilities_users,
        urnietfparamsscimschemasoracleidcsextensiondb_credentials_users=__ret__.urnietfparamsscimschemasoracleidcsextensiondb_credentials_users,
        urnietfparamsscimschemasoracleidcsextensiondb_user_users=__ret__.urnietfparamsscimschemasoracleidcsextensiondb_user_users,
        urnietfparamsscimschemasoracleidcsextensionkerberos_user_users=__ret__.urnietfparamsscimschemasoracleidcsextensionkerberos_user_users,
        urnietfparamsscimschemasoracleidcsextensionmfa_users=__ret__.urnietfparamsscimschemasoracleidcsextensionmfa_users,
        urnietfparamsscimschemasoracleidcsextensionpassword_state_users=__ret__.urnietfparamsscimschemasoracleidcsextensionpassword_state_users,
        urnietfparamsscimschemasoracleidcsextensionpasswordless_users=__ret__.urnietfparamsscimschemasoracleidcsextensionpasswordless_users,
        urnietfparamsscimschemasoracleidcsextensionposix_users=__ret__.urnietfparamsscimschemasoracleidcsextensionposix_users,
        urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users=__ret__.urnietfparamsscimschemasoracleidcsextensionsecurity_questions_users,
        urnietfparamsscimschemasoracleidcsextensionself_change_users=__ret__.urnietfparamsscimschemasoracleidcsextensionself_change_users,
        urnietfparamsscimschemasoracleidcsextensionself_registration_users=__ret__.urnietfparamsscimschemasoracleidcsextensionself_registration_users,
        urnietfparamsscimschemasoracleidcsextensionsff_users=__ret__.urnietfparamsscimschemasoracleidcsextensionsff_users,
        urnietfparamsscimschemasoracleidcsextensionsocial_account_users=__ret__.urnietfparamsscimschemasoracleidcsextensionsocial_account_users,
        urnietfparamsscimschemasoracleidcsextensionterms_of_use_users=__ret__.urnietfparamsscimschemasoracleidcsextensionterms_of_use_users,
        urnietfparamsscimschemasoracleidcsextensionuser_credentials_users=__ret__.urnietfparamsscimschemasoracleidcsextensionuser_credentials_users,
        urnietfparamsscimschemasoracleidcsextensionuser_state_users=__ret__.urnietfparamsscimschemasoracleidcsextensionuser_state_users,
        urnietfparamsscimschemasoracleidcsextensionuser_users=__ret__.urnietfparamsscimschemasoracleidcsextensionuser_users,
        user_id=__ret__.user_id,
        user_name=__ret__.user_name,
        user_type=__ret__.user_type,
        x509certificates=__ret__.x509certificates)


@_utilities.lift_output_func(get_domains_user)
def get_domains_user_output(attribute_sets: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                            attributes: Optional[pulumi.Input[Optional[str]]] = None,
                            authorization: Optional[pulumi.Input[Optional[str]]] = None,
                            idcs_endpoint: Optional[pulumi.Input[str]] = None,
                            resource_type_schema_version: Optional[pulumi.Input[Optional[str]]] = None,
                            user_id: Optional[pulumi.Input[str]] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDomainsUserResult]:
    """
    This data source provides details about a specific User resource in Oracle Cloud Infrastructure Identity Domains service.

    Get a User

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_user = oci.Identity.get_domains_user(idcs_endpoint=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        user_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        attribute_sets=[],
        attributes="",
        authorization=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        resource_type_schema_version=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param Sequence[str] attribute_sets: A multi-valued list of strings indicating the return type of attribute definition. The specified set of attributes can be fetched by the return type of the attribute. One or more values can be given together to fetch more than one group of attributes. If 'attributes' query parameter is also available, union of the two is fetched. Valid values - all, always, never, request, default. Values are case-insensitive.
    :param str attributes: A comma-delimited string that specifies the names of resource attributes that should be returned in the response. By default, a response that contains resource attributes contains only attributes that are defined in the schema for that resource type as returned=always or returned=default. An attribute that is defined as returned=request is returned in a response only if the request specifies its name in the value of this query parameter. If a request specifies this query parameter, the response contains the attributes that this query parameter specifies, as well as any attribute that is defined as returned=always.
    :param str authorization: The Authorization field value consists of credentials containing the authentication information of the user agent for the realm of the resource being requested.
    :param str idcs_endpoint: The basic endpoint for the identity domain
    :param str resource_type_schema_version: An endpoint-specific schema version number to use in the Request. Allowed version values are Earliest Version or Latest Version as specified in each REST API endpoint description, or any sequential number inbetween. All schema attributes/body parameters are a part of version 1. After version 1, any attributes added or deprecated will be tagged with the version that they were added to or deprecated in. If no version is provided, the latest schema version is returned.
    :param str user_id: ID of the resource
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetPreauthrequestsResult',
    'AwaitableGetPreauthrequestsResult',
    'get_preauthrequests',
    'get_preauthrequests_output',
]

@pulumi.output_type
class GetPreauthrequestsResult:
    """
    A collection of values returned by getPreauthrequests.
    """
    def __init__(__self__, bucket=None, filters=None, id=None, namespace=None, object_name_prefix=None, preauthenticated_requests=None):
        if bucket and not isinstance(bucket, str):
            raise TypeError("Expected argument 'bucket' to be a str")
        pulumi.set(__self__, "bucket", bucket)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)
        if object_name_prefix and not isinstance(object_name_prefix, str):
            raise TypeError("Expected argument 'object_name_prefix' to be a str")
        pulumi.set(__self__, "object_name_prefix", object_name_prefix)
        if preauthenticated_requests and not isinstance(preauthenticated_requests, list):
            raise TypeError("Expected argument 'preauthenticated_requests' to be a list")
        pulumi.set(__self__, "preauthenticated_requests", preauthenticated_requests)

    @property
    @pulumi.getter
    def bucket(self) -> str:
        """
        The name of the bucket.  Example: `my-new-bucket1`
        """
        return pulumi.get(self, "bucket")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetPreauthrequestsFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def namespace(self) -> str:
        """
        The Object Storage namespace used for the request.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="objectNamePrefix")
    def object_name_prefix(self) -> Optional[str]:
        return pulumi.get(self, "object_name_prefix")

    @property
    @pulumi.getter(name="preauthenticatedRequests")
    def preauthenticated_requests(self) -> Sequence['outputs.GetPreauthrequestsPreauthenticatedRequestResult']:
        """
        The list of preauthenticated_requests.
        """
        return pulumi.get(self, "preauthenticated_requests")


class AwaitableGetPreauthrequestsResult(GetPreauthrequestsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetPreauthrequestsResult(
            bucket=self.bucket,
            filters=self.filters,
            id=self.id,
            namespace=self.namespace,
            object_name_prefix=self.object_name_prefix,
            preauthenticated_requests=self.preauthenticated_requests)


def get_preauthrequests(bucket: Optional[str] = None,
                        filters: Optional[Sequence[pulumi.InputType['GetPreauthrequestsFilterArgs']]] = None,
                        namespace: Optional[str] = None,
                        object_name_prefix: Optional[str] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetPreauthrequestsResult:
    """
    This data source provides the list of Preauthenticated Requests in Oracle Cloud Infrastructure Object Storage service.

    Lists pre-authenticated requests for the bucket.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_preauthenticated_requests = oci.ObjectStorage.get_preauthrequests(bucket=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        namespace=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        object_name_prefix=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str bucket: The name of the bucket. Avoid entering confidential information. Example: `my-new-bucket1`
    :param str namespace: The Object Storage namespace used for the request.
    :param str object_name_prefix: User-specified object name prefixes can be used to query and return a list of pre-authenticated requests.
    """
    __args__ = dict()
    __args__['bucket'] = bucket
    __args__['filters'] = filters
    __args__['namespace'] = namespace
    __args__['objectNamePrefix'] = object_name_prefix
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:ObjectStorage/getPreauthrequests:getPreauthrequests', __args__, opts=opts, typ=GetPreauthrequestsResult).value

    return AwaitableGetPreauthrequestsResult(
        bucket=__ret__.bucket,
        filters=__ret__.filters,
        id=__ret__.id,
        namespace=__ret__.namespace,
        object_name_prefix=__ret__.object_name_prefix,
        preauthenticated_requests=__ret__.preauthenticated_requests)


@_utilities.lift_output_func(get_preauthrequests)
def get_preauthrequests_output(bucket: Optional[pulumi.Input[str]] = None,
                               filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetPreauthrequestsFilterArgs']]]]] = None,
                               namespace: Optional[pulumi.Input[str]] = None,
                               object_name_prefix: Optional[pulumi.Input[Optional[str]]] = None,
                               opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetPreauthrequestsResult]:
    """
    This data source provides the list of Preauthenticated Requests in Oracle Cloud Infrastructure Object Storage service.

    Lists pre-authenticated requests for the bucket.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_preauthenticated_requests = oci.ObjectStorage.get_preauthrequests(bucket=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        namespace=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        object_name_prefix=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str bucket: The name of the bucket. Avoid entering confidential information. Example: `my-new-bucket1`
    :param str namespace: The Object Storage namespace used for the request.
    :param str object_name_prefix: User-specified object name prefixes can be used to query and return a list of pre-authenticated requests.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetInvoicesInvoiceLinesResult',
    'AwaitableGetInvoicesInvoiceLinesResult',
    'get_invoices_invoice_lines',
    'get_invoices_invoice_lines_output',
]

@pulumi.output_type
class GetInvoicesInvoiceLinesResult:
    """
    A collection of values returned by getInvoicesInvoiceLines.
    """
    def __init__(__self__, compartment_id=None, filters=None, id=None, internal_invoice_id=None, invoice_line_collections=None, osp_home_region=None):
        if compartment_id and not isinstance(compartment_id, str):
            raise TypeError("Expected argument 'compartment_id' to be a str")
        pulumi.set(__self__, "compartment_id", compartment_id)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if internal_invoice_id and not isinstance(internal_invoice_id, str):
            raise TypeError("Expected argument 'internal_invoice_id' to be a str")
        pulumi.set(__self__, "internal_invoice_id", internal_invoice_id)
        if invoice_line_collections and not isinstance(invoice_line_collections, list):
            raise TypeError("Expected argument 'invoice_line_collections' to be a list")
        pulumi.set(__self__, "invoice_line_collections", invoice_line_collections)
        if osp_home_region and not isinstance(osp_home_region, str):
            raise TypeError("Expected argument 'osp_home_region' to be a str")
        pulumi.set(__self__, "osp_home_region", osp_home_region)

    @property
    @pulumi.getter(name="compartmentId")
    def compartment_id(self) -> str:
        return pulumi.get(self, "compartment_id")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetInvoicesInvoiceLinesFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="internalInvoiceId")
    def internal_invoice_id(self) -> str:
        return pulumi.get(self, "internal_invoice_id")

    @property
    @pulumi.getter(name="invoiceLineCollections")
    def invoice_line_collections(self) -> Sequence['outputs.GetInvoicesInvoiceLinesInvoiceLineCollectionResult']:
        """
        The list of invoice_line_collection.
        """
        return pulumi.get(self, "invoice_line_collections")

    @property
    @pulumi.getter(name="ospHomeRegion")
    def osp_home_region(self) -> str:
        return pulumi.get(self, "osp_home_region")


class AwaitableGetInvoicesInvoiceLinesResult(GetInvoicesInvoiceLinesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetInvoicesInvoiceLinesResult(
            compartment_id=self.compartment_id,
            filters=self.filters,
            id=self.id,
            internal_invoice_id=self.internal_invoice_id,
            invoice_line_collections=self.invoice_line_collections,
            osp_home_region=self.osp_home_region)


def get_invoices_invoice_lines(compartment_id: Optional[str] = None,
                               filters: Optional[Sequence[pulumi.InputType['GetInvoicesInvoiceLinesFilterArgs']]] = None,
                               internal_invoice_id: Optional[str] = None,
                               osp_home_region: Optional[str] = None,
                               opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetInvoicesInvoiceLinesResult:
    """
    This data source provides the list of Invoices Invoice Lines in Oracle Cloud Infrastructure Osp Gateway service.

    Returns the invoice product list by invoice id

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_invoices_invoice_lines = oci.OspGateway.get_invoices_invoice_lines(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        internal_invoice_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        osp_home_region=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    :param str internal_invoice_id: The identifier of the invoice.
    :param str osp_home_region: The home region's public name of the logged in user.
    """
    __args__ = dict()
    __args__['compartmentId'] = compartment_id
    __args__['filters'] = filters
    __args__['internalInvoiceId'] = internal_invoice_id
    __args__['ospHomeRegion'] = osp_home_region
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:OspGateway/getInvoicesInvoiceLines:getInvoicesInvoiceLines', __args__, opts=opts, typ=GetInvoicesInvoiceLinesResult).value

    return AwaitableGetInvoicesInvoiceLinesResult(
        compartment_id=__ret__.compartment_id,
        filters=__ret__.filters,
        id=__ret__.id,
        internal_invoice_id=__ret__.internal_invoice_id,
        invoice_line_collections=__ret__.invoice_line_collections,
        osp_home_region=__ret__.osp_home_region)


@_utilities.lift_output_func(get_invoices_invoice_lines)
def get_invoices_invoice_lines_output(compartment_id: Optional[pulumi.Input[str]] = None,
                                      filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetInvoicesInvoiceLinesFilterArgs']]]]] = None,
                                      internal_invoice_id: Optional[pulumi.Input[str]] = None,
                                      osp_home_region: Optional[pulumi.Input[str]] = None,
                                      opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetInvoicesInvoiceLinesResult]:
    """
    This data source provides the list of Invoices Invoice Lines in Oracle Cloud Infrastructure Osp Gateway service.

    Returns the invoice product list by invoice id

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_invoices_invoice_lines = oci.OspGateway.get_invoices_invoice_lines(compartment_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        internal_invoice_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        osp_home_region=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str compartment_id: The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment.
    :param str internal_invoice_id: The identifier of the invoice.
    :param str osp_home_region: The home region's public name of the logged in user.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetBdsInstanceApiKeysResult',
    'AwaitableGetBdsInstanceApiKeysResult',
    'get_bds_instance_api_keys',
    'get_bds_instance_api_keys_output',
]

@pulumi.output_type
class GetBdsInstanceApiKeysResult:
    """
    A collection of values returned by getBdsInstanceApiKeys.
    """
    def __init__(__self__, bds_api_keys=None, bds_instance_id=None, display_name=None, filters=None, id=None, state=None, user_id=None):
        if bds_api_keys and not isinstance(bds_api_keys, list):
            raise TypeError("Expected argument 'bds_api_keys' to be a list")
        pulumi.set(__self__, "bds_api_keys", bds_api_keys)
        if bds_instance_id and not isinstance(bds_instance_id, str):
            raise TypeError("Expected argument 'bds_instance_id' to be a str")
        pulumi.set(__self__, "bds_instance_id", bds_instance_id)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if user_id and not isinstance(user_id, str):
            raise TypeError("Expected argument 'user_id' to be a str")
        pulumi.set(__self__, "user_id", user_id)

    @property
    @pulumi.getter(name="bdsApiKeys")
    def bds_api_keys(self) -> Sequence['outputs.GetBdsInstanceApiKeysBdsApiKeyResult']:
        return pulumi.get(self, "bds_api_keys")

    @property
    @pulumi.getter(name="bdsInstanceId")
    def bds_instance_id(self) -> str:
        return pulumi.get(self, "bds_instance_id")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[str]:
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetBdsInstanceApiKeysFilterResult']]:
        return pulumi.get(self, "filters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def state(self) -> Optional[str]:
        """
        The current status of the API key.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> Optional[str]:
        """
        The user OCID for which this API key was created.
        """
        return pulumi.get(self, "user_id")


class AwaitableGetBdsInstanceApiKeysResult(GetBdsInstanceApiKeysResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetBdsInstanceApiKeysResult(
            bds_api_keys=self.bds_api_keys,
            bds_instance_id=self.bds_instance_id,
            display_name=self.display_name,
            filters=self.filters,
            id=self.id,
            state=self.state,
            user_id=self.user_id)


def get_bds_instance_api_keys(bds_instance_id: Optional[str] = None,
                              display_name: Optional[str] = None,
                              filters: Optional[Sequence[pulumi.InputType['GetBdsInstanceApiKeysFilterArgs']]] = None,
                              state: Optional[str] = None,
                              user_id: Optional[str] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetBdsInstanceApiKeysResult:
    """
    This data source provides details about a specific Bds Instance Api Key resource in Oracle Cloud Infrastructure Big Data Service service.

    Returns the user's API key information for the given ID.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_bds_instance_api_key = oci.BigDataService.get_bds_instance_api_key(api_key_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        bds_instance_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str bds_instance_id: The OCID of the cluster.
    :param str state: The current status of the API key.
    :param str user_id: The user OCID for which this API key was created.
    """
    __args__ = dict()
    __args__['bdsInstanceId'] = bds_instance_id
    __args__['displayName'] = display_name
    __args__['filters'] = filters
    __args__['state'] = state
    __args__['userId'] = user_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('oci:BigDataService/getBdsInstanceApiKeys:getBdsInstanceApiKeys', __args__, opts=opts, typ=GetBdsInstanceApiKeysResult).value

    return AwaitableGetBdsInstanceApiKeysResult(
        bds_api_keys=__ret__.bds_api_keys,
        bds_instance_id=__ret__.bds_instance_id,
        display_name=__ret__.display_name,
        filters=__ret__.filters,
        id=__ret__.id,
        state=__ret__.state,
        user_id=__ret__.user_id)


@_utilities.lift_output_func(get_bds_instance_api_keys)
def get_bds_instance_api_keys_output(bds_instance_id: Optional[pulumi.Input[str]] = None,
                                     display_name: Optional[pulumi.Input[Optional[str]]] = None,
                                     filters: Optional[pulumi.Input[Optional[Sequence[pulumi.InputType['GetBdsInstanceApiKeysFilterArgs']]]]] = None,
                                     state: Optional[pulumi.Input[Optional[str]]] = None,
                                     user_id: Optional[pulumi.Input[Optional[str]]] = None,
                                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetBdsInstanceApiKeysResult]:
    """
    This data source provides details about a specific Bds Instance Api Key resource in Oracle Cloud Infrastructure Big Data Service service.

    Returns the user's API key information for the given ID.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_oci as oci

    test_bds_instance_api_key = oci.BigDataService.get_bds_instance_api_key(api_key_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference),
        bds_instance_id=%!v(PANIC=Format method: runtime error: invalid memory address or nil pointer dereference))
    ```


    :param str bds_instance_id: The OCID of the cluster.
    :param str state: The current status of the API key.
    :param str user_id: The user OCID for which this API key was created.
    """
    ...
